==> wilderflowers-1.0.0%2B1.21.10-fabric-sources.jar: cc/cassian/wilderflowers/blocks/FlowerGarlandBlock.java <==
package cc.cassian.wilderflowers.blocks;

import com.google.common.collect.ImmutableMap;
import net.minecraft.class_10225;
import net.minecraft.class_156;
import net.minecraft.class_1750;
import net.minecraft.class_1922;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2415;
import net.minecraft.class_2429;
import net.minecraft.class_247;
import net.minecraft.class_2470;
import net.minecraft.class_259;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_3726;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.world.level.*;
import net.minecraft.world.level.block.*;
import net.minecraft.world.level.block.state.properties.*;
import net.minecraft.world.phys.shapes.*;
import org.jetbrains.annotations.Nullable;

import java.awt.*;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public class FlowerGarlandBlock extends class_2248 {
    public static final class_2746 NORTH = class_2741.field_12489;
    public static final class_2746 EAST = class_2741.field_12487;
    public static final class_2746 SOUTH = class_2741.field_12540;
    public static final class_2746 WEST = class_2741.field_12527;
    public static final Map<class_2350, class_2746> PROPERTY_BY_DIRECTION = class_2429.field_11329.entrySet().stream().filter((entry) -> (entry.getKey() != class_2350.field_11033 && entry.getKey() != class_2350.field_11036)).collect(class_156.method_664());;
    private static final class_265 WEST_AABB = class_2248.method_9541(0.0F, 9.0F, 0.0F, 1.0F, 15.0F, 16.0F);
    private static final class_265 EAST_AABB = class_2248.method_9541(15.0F, 9.0F, 0.0F, 16.0F, 15.0F, 16.0F);
    private static final class_265 NORTH_AABB = class_2248.method_9541(0.0F, 9.0F, 0.0F, 16.0F, 15.0F, 1.0F);
    private static final class_265 SOUTH_AABB = class_2248.method_9541(0.0F, 9.0F, 15.0F, 16.0F, 15.0F, 16.0F);
    private static final class_265 SUPPORT_SHAPE = class_2248.method_9541(0.0F, 13.0F, 0.0F, 16.0F, 16.0F, 16.0F);
    private final Map<class_2680, class_265> shapesCache;

    public FlowerGarlandBlock(class_2251 properties) {
        super(properties);
        this.method_9590(this.field_10647.method_11664()
                .method_11657(NORTH, false)
                .method_11657(EAST, false)
                .method_11657(SOUTH, false)
                .method_11657(WEST, false));
        this.shapesCache = ImmutableMap.copyOf(this.field_10647.method_11662().stream().collect(Collectors.toMap(Function.identity(), FlowerGarlandBlock::calculateShape)));
    }

    private static class_265 calculateShape(class_2680 state) {
        class_265 voxelshape = class_259.method_1073();
        if (state.method_11654(NORTH)) {
            voxelshape = class_259.method_1084(voxelshape, NORTH_AABB);
        }

        if (state.method_11654(SOUTH)) {
            voxelshape = class_259.method_1084(voxelshape, SOUTH_AABB);
        }

        if (state.method_11654(EAST)) {
            voxelshape = class_259.method_1084(voxelshape, EAST_AABB);
        }

        if (state.method_11654(WEST)) {
            voxelshape = class_259.method_1084(voxelshape, WEST_AABB);
        }

        return voxelshape.method_1110() ? class_259.method_1077() : voxelshape;
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 level, class_2338 pos, class_3726 context) {
        return this.shapesCache.get(state);
    }

    //? if <1.21.2 {
    /*@Override
    public boolean propagatesSkylightDown(BlockState state, BlockGetter level, BlockPos pos) {
        return true;
    }
    *///?} else {

    @Override
    protected boolean method_9579(class_2680 state) {
        return true;
    }
    //?}

    @Override
    public boolean method_9558(class_2680 state, class_4538 level, class_2338 pos) {
        return this.hasFaces(this.getUpdatedState(state, level, pos));
    }

    private boolean hasFaces(class_2680 state) {
        return this.countFaces(state) > 0;
    }

    private int countFaces(class_2680 state) {
        int i = 0;

        for(class_2746 booleanproperty : PROPERTY_BY_DIRECTION.values()) {
            if (state.method_11654(booleanproperty)) {
                ++i;
            }
        }

        return i;
    }

    private boolean canSupportAtFace(class_1922 level, class_2338 pos, class_2350 direction) {
        if (direction.method_10166() == class_2350.class_2351.field_11052) {
            return false;
        } else {
            class_2338 blockpos = pos.method_10093(direction);
            return isAcceptableNeighbour(level, blockpos, direction);
        }
    }

    public static boolean isAcceptableNeighbour(class_1922 blockReader, class_2338 neighborPos, class_2350 attachedFace) {
        class_2680 state = blockReader.method_8320(neighborPos);

        class_265 neighborSupportFaceShape = state.method_26222(blockReader, neighborPos).method_20538(attachedFace.method_10153());
        class_265 neighborCollisionFaceShape = state.method_26220(blockReader, neighborPos).method_20538(attachedFace.method_10153());

        // Check that neighbour shape covers support shape
        return !class_259.method_1074(neighborCollisionFaceShape, SUPPORT_SHAPE, class_247.field_16893) || !class_259.method_1074(neighborSupportFaceShape, SUPPORT_SHAPE, class_247.field_16893);
    }

    private class_2680 getUpdatedState(class_2680 state, class_1922 level, class_2338 pos) {
        class_2338 blockpos = pos.method_10084();
        class_2680 blockstate = null;

        for(class_2350 direction : class_2350.class_2353.field_11062) {
            class_2746 booleanproperty = getPropertyForFace(direction);
            if (state.method_11654(booleanproperty)) {
                boolean flag = this.canSupportAtFace(level, pos, direction);
                if (!flag) {
                    if (blockstate == null) {
                        blockstate = level.method_8320(blockpos);
                    }

                    flag = blockstate.method_27852(this) && blockstate.method_11654(booleanproperty);
                }

                state = state.method_11657(booleanproperty, flag);
            }
        }

        return state;
    }

    //? if <1.21.2 {
    /*@Override
    public BlockState updateShape(BlockState state, Direction direction, BlockState neighborState, LevelAccessor level, BlockPos currentPos, BlockPos neighborPos) {
        if (direction.getAxis() == Direction.Axis.Y) {
            return super.updateShape(state, direction, neighborState, level, currentPos, neighborPos);
        } else {
            BlockState blockstate = this.getUpdatedState(state, level, currentPos);
            return !this.hasFaces(blockstate) ? Blocks.AIR.defaultBlockState() : blockstate;
        }
    }
    *///?} else {

    @Override
    protected class_2680 method_9559(class_2680 state, class_4538 level, class_10225 scheduledTickAccess, class_2338 currentPos, class_2350 direction, class_2338 neighborPos, class_2680 neighborState, class_5819 random
    ) {
        if (direction.method_10166() == class_2350.class_2351.field_11052) {
            return super.method_9559(state, level, scheduledTickAccess, currentPos, direction, neighborPos, neighborState, random);
        } else {
            class_2680 blockstate = this.getUpdatedState(state, level, currentPos);
            return !this.hasFaces(blockstate) ? class_2246.field_10124.method_9564() : blockstate;
        }
    }
    //?}

    @Override
    public boolean method_9616(class_2680 blockState, class_1750 blockPlaceContext) {
        if (!blockPlaceContext.method_8046() && blockPlaceContext.method_8041().method_31574(this.method_8389())) {
            return this.countFaces(blockState) < PROPERTY_BY_DIRECTION.size();
        } else {
            return super.method_9616(blockState, blockPlaceContext);
        }
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        class_2680 blockstate = context.method_8045().method_8320(context.method_8037());
        boolean flag = blockstate.method_27852(this);
        class_2680 blockstate1 = flag ? blockstate : this.method_9564();

        for(class_2350 direction : context.method_7718()) {
            if (direction.method_10166() != class_2350.class_2351.field_11052) {
                class_2746 booleanproperty = getPropertyForFace(direction);
                boolean flag1 = flag && blockstate.method_11654(booleanproperty);
                if (!flag1 && this.canSupportAtFace(context.method_8045(), context.method_8037(), direction)) {
                    return blockstate1.method_11657(booleanproperty, true);
                }
            }
        }

        return flag ? blockstate1 : null;
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
        builder.method_11667(NORTH, EAST, SOUTH, WEST);
    }

    @Override
    public class_2680 method_9598(class_2680 state, class_2470 rotation) {
        switch (rotation) {
            case field_11464 -> {
                return (((state.method_11657(NORTH, state.method_11654(SOUTH))).method_11657(EAST, state.method_11654(WEST))).method_11657(SOUTH, state.method_11654(NORTH))).method_11657(WEST, state.method_11654(EAST));
            }
            case field_11465 -> {
                return (((state.method_11657(NORTH, state.method_11654(EAST))).method_11657(EAST, state.method_11654(SOUTH))).method_11657(SOUTH, state.method_11654(WEST))).method_11657(WEST, state.method_11654(NORTH));
            }
            case field_11463 -> {
                return (((state.method_11657(NORTH, state.method_11654(WEST))).method_11657(EAST, state.method_11654(NORTH))).method_11657(SOUTH, state.method_11654(EAST))).method_11657(WEST, state.method_11654(SOUTH));
            }
            default -> {
                return state;
            }
        }
    }

    @Override
    public class_2680 method_9569(class_2680 state, class_2415 mirror) {
        switch (mirror) {
            case field_11300 -> {
                return (state.method_11657(NORTH, state.method_11654(SOUTH))).method_11657(SOUTH, state.method_11654(NORTH));
            }
            case field_11301 -> {
                return (state.method_11657(EAST, state.method_11654(WEST))).method_11657(WEST, state.method_11654(EAST));
            }
            default -> {
                return super.method_9569(state, mirror);
            }
        }
    }

    public static class_2746 getPropertyForFace(class_2350 face) {
        return PROPERTY_BY_DIRECTION.get(face);
    }
}

==> wilderflowers-1.0.2%2B1.20.1-fabric-sources.jar: cc/cassian/wilderflowers/blocks/FlowerGarlandBlock.java <==
package cc.cassian.wilderflowers.blocks;

import com.google.common.collect.ImmutableMap;
import net.minecraft.class_156;
import net.minecraft.class_1750;
import net.minecraft.class_1922;
import net.minecraft.class_1936;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2415;
import net.minecraft.class_2429;
import net.minecraft.class_247;
import net.minecraft.class_2470;
import net.minecraft.class_259;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_3726;
import net.minecraft.class_4538;
import net.minecraft.world.level.*;
import net.minecraft.world.level.block.*;
import net.minecraft.world.level.block.state.properties.*;
import net.minecraft.world.phys.shapes.*;
import org.jetbrains.annotations.Nullable;

import java.awt.*;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public class FlowerGarlandBlock extends class_2248 {
    public static final class_2746 NORTH = class_2741.field_12489;
    public static final class_2746 EAST = class_2741.field_12487;
    public static final class_2746 SOUTH = class_2741.field_12540;
    public static final class_2746 WEST = class_2741.field_12527;
    public static final Map<class_2350, class_2746> PROPERTY_BY_DIRECTION = class_2429.field_11329.entrySet().stream().filter((entry) -> (entry.getKey() != class_2350.field_11033 && entry.getKey() != class_2350.field_11036)).collect(class_156.method_664());;
    private static final class_265 WEST_AABB = class_2248.method_9541(0.0F, 9.0F, 0.0F, 1.0F, 15.0F, 16.0F);
    private static final class_265 EAST_AABB = class_2248.method_9541(15.0F, 9.0F, 0.0F, 16.0F, 15.0F, 16.0F);
    private static final class_265 NORTH_AABB = class_2248.method_9541(0.0F, 9.0F, 0.0F, 16.0F, 15.0F, 1.0F);
    private static final class_265 SOUTH_AABB = class_2248.method_9541(0.0F, 9.0F, 15.0F, 16.0F, 15.0F, 16.0F);
    private static final class_265 SUPPORT_SHAPE = class_2248.method_9541(0.0F, 13.0F, 0.0F, 16.0F, 16.0F, 16.0F);
    private final Map<class_2680, class_265> shapesCache;

    public FlowerGarlandBlock(class_2251 properties) {
        super(properties);
        this.method_9590(this.field_10647.method_11664()
                .method_11657(NORTH, false)
                .method_11657(EAST, false)
                .method_11657(SOUTH, false)
                .method_11657(WEST, false));
        this.shapesCache = ImmutableMap.copyOf(this.field_10647.method_11662().stream().collect(Collectors.toMap(Function.identity(), FlowerGarlandBlock::calculateShape)));
    }

    private static class_265 calculateShape(class_2680 state) {
        class_265 voxelshape = class_259.method_1073();
        if (state.method_11654(NORTH)) {
            voxelshape = class_259.method_1084(voxelshape, NORTH_AABB);
        }

        if (state.method_11654(SOUTH)) {
            voxelshape = class_259.method_1084(voxelshape, SOUTH_AABB);
        }

        if (state.method_11654(EAST)) {
            voxelshape = class_259.method_1084(voxelshape, EAST_AABB);
        }

        if (state.method_11654(WEST)) {
            voxelshape = class_259.method_1084(voxelshape, WEST_AABB);
        }

        return voxelshape.method_1110() ? class_259.method_1077() : voxelshape;
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 level, class_2338 pos, class_3726 context) {
        return this.shapesCache.get(state);
    }

    //? if <1.21.2 {
    @Override
    public boolean method_9579(class_2680 state, class_1922 level, class_2338 pos) {
        return true;
    }
    //?} else {

    /*@Override
    protected boolean propagatesSkylightDown(BlockState state) {
        return true;
    }
    *///?}

    @Override
    public boolean method_9558(class_2680 state, class_4538 level, class_2338 pos) {
        return this.hasFaces(this.getUpdatedState(state, level, pos));
    }

    private boolean hasFaces(class_2680 state) {
        return this.countFaces(state) > 0;
    }

    private int countFaces(class_2680 state) {
        int i = 0;

        for(class_2746 booleanproperty : PROPERTY_BY_DIRECTION.values()) {
            if (state.method_11654(booleanproperty)) {
                ++i;
            }
        }

        return i;
    }

    private boolean canSupportAtFace(class_1922 level, class_2338 pos, class_2350 direction) {
        if (direction.method_10166() == class_2350.class_2351.field_11052) {
            return false;
        } else {
            class_2338 blockpos = pos.method_10093(direction);
            return isAcceptableNeighbour(level, blockpos, direction);
        }
    }

    public static boolean isAcceptableNeighbour(class_1922 blockReader, class_2338 neighborPos, class_2350 attachedFace) {
        class_2680 state = blockReader.method_8320(neighborPos);

        class_265 neighborSupportFaceShape = state.method_26222(blockReader, neighborPos).method_20538(attachedFace.method_10153());
        class_265 neighborCollisionFaceShape = state.method_26220(blockReader, neighborPos).method_20538(attachedFace.method_10153());

        // Check that neighbour shape covers support shape
        return !class_259.method_1074(neighborCollisionFaceShape, SUPPORT_SHAPE, class_247.field_16893) || !class_259.method_1074(neighborSupportFaceShape, SUPPORT_SHAPE, class_247.field_16893);
    }

    private class_2680 getUpdatedState(class_2680 state, class_1922 level, class_2338 pos) {
        class_2338 blockpos = pos.method_10084();
        class_2680 blockstate = null;

        for(class_2350 direction : class_2350.class_2353.field_11062) {
            class_2746 booleanproperty = getPropertyForFace(direction);
            if (state.method_11654(booleanproperty)) {
                boolean flag = this.canSupportAtFace(level, pos, direction);
                if (!flag) {
                    if (blockstate == null) {
                        blockstate = level.method_8320(blockpos);
                    }

                    flag = blockstate.method_27852(this) && blockstate.method_11654(booleanproperty);
                }

                state = state.method_11657(booleanproperty, flag);
            }
        }

        return state;
    }

    //? if <1.21.2 {
    @Override
    public class_2680 method_9559(class_2680 state, class_2350 direction, class_2680 neighborState, class_1936 level, class_2338 currentPos, class_2338 neighborPos) {
        if (direction.method_10166() == class_2350.class_2351.field_11052) {
            return super.method_9559(state, direction, neighborState, level, currentPos, neighborPos);
        } else {
            class_2680 blockstate = this.getUpdatedState(state, level, currentPos);
            return !this.hasFaces(blockstate) ? class_2246.field_10124.method_9564() : blockstate;
        }
    }
    //?} else {

    /*@Override
    protected BlockState updateShape(BlockState state, LevelReader level, ScheduledTickAccess scheduledTickAccess, BlockPos currentPos, Direction direction, BlockPos neighborPos, BlockState neighborState, RandomSource random
    ) {
        if (direction.getAxis() == Direction.Axis.Y) {
            return super.updateShape(state, level, scheduledTickAccess, currentPos, direction, neighborPos, neighborState, random);
        } else {
            BlockState blockstate = this.getUpdatedState(state, level, currentPos);
            return !this.hasFaces(blockstate) ? Blocks.AIR.defaultBlockState() : blockstate;
        }
    }
    *///?}

    @Override
    public boolean method_9616(class_2680 blockState, class_1750 blockPlaceContext) {
        if (!blockPlaceContext.method_8046() && blockPlaceContext.method_8041().method_31574(this.method_8389())) {
            return this.countFaces(blockState) < PROPERTY_BY_DIRECTION.size();
        } else {
            return super.method_9616(blockState, blockPlaceContext);
        }
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        class_2680 blockstate = context.method_8045().method_8320(context.method_8037());
        boolean flag = blockstate.method_27852(this);
        class_2680 blockstate1 = flag ? blockstate : this.method_9564();

        for(class_2350 direction : context.method_7718()) {
            if (direction.method_10166() != class_2350.class_2351.field_11052) {
                class_2746 booleanproperty = getPropertyForFace(direction);
                boolean flag1 = flag && blockstate.method_11654(booleanproperty);
                if (!flag1 && this.canSupportAtFace(context.method_8045(), context.method_8037(), direction)) {
                    return blockstate1.method_11657(booleanproperty, true);
                }
            }
        }

        return flag ? blockstate1 : null;
    }

    @Override
    protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
        builder.method_11667(NORTH, EAST, SOUTH, WEST);
    }

    @Override
    public class_2680 method_9598(class_2680 state, class_2470 rotation) {
        switch (rotation) {
            case field_11464 -> {
                return (((state.method_11657(NORTH, state.method_11654(SOUTH))).method_11657(EAST, state.method_11654(WEST))).method_11657(SOUTH, state.method_11654(NORTH))).method_11657(WEST, state.method_11654(EAST));
            }
            case field_11465 -> {
                return (((state.method_11657(NORTH, state.method_11654(EAST))).method_11657(EAST, state.method_11654(SOUTH))).method_11657(SOUTH, state.method_11654(WEST))).method_11657(WEST, state.method_11654(NORTH));
            }
            case field_11463 -> {
                return (((state.method_11657(NORTH, state.method_11654(WEST))).method_11657(EAST, state.method_11654(NORTH))).method_11657(SOUTH, state.method_11654(EAST))).method_11657(WEST, state.method_11654(SOUTH));
            }
            default -> {
                return state;
            }
        }
    }

    @Override
    public class_2680 method_9569(class_2680 state, class_2415 mirror) {
        switch (mirror) {
            case field_11300 -> {
                return (state.method_11657(NORTH, state.method_11654(SOUTH))).method_11657(SOUTH, state.method_11654(NORTH));
            }
            case field_11301 -> {
                return (state.method_11657(EAST, state.method_11654(WEST))).method_11657(WEST, state.method_11654(EAST));
            }
            default -> {
                return super.method_9569(state, mirror);
            }
        }
    }

    public static class_2746 getPropertyForFace(class_2350 face) {
        return PROPERTY_BY_DIRECTION.get(face);
    }
}
